# Python imports
import os

# Lib imports
from gi.repository import GLib

# Application imports
from libs.dto.lsp_messages import get_message_obj, didopen_notification, completion_request, didchange_notification



class LSPControllerEvents:
    def send_initialize_message(self, init_ops: str, workspace_file: str, workspace_uri: str):
        folder_name = os.path.basename(workspace_file)

        self._init_params["processId"]        = settings_manager.get_app_pid()
        self._init_params["rootPath"]         = workspace_file
        self._init_params["rootUri"]          = workspace_uri
        self._init_params["workspaceFolders"] = [
            {
                "name": folder_name,
                "uri": workspace_uri
            }
        ]

        self._init_params["initializationOptions"] = get_message_obj(init_ops)
        self.send_request("initialize", self._init_params)

    def send_initialized_message(self):
        self.send_notification("initialized")

    def _lsp_did_open(self, data: dict):
        method = data["method"]
        params = didopen_notification["params"]

        params["textDocument"]["uri"]        = data["uri"]
        params["textDocument"]["languageId"] = data["language_id"]
        params["textDocument"]["text"]       = data["text"]

        GLib.idle_add( self.send_notification, method, params )

    def _lsp_did_save(self, data: dict):
        # GLib.idle_add( self.send_notification, method, params )
        ...

    def _lsp_did_close(self, data: dict):
        # GLib.idle_add( self.send_notification, method, params )
        ...

    def _lsp_did_change(self, data: dict):
        method = data["method"]
        params = didchange_notification["params"]

        params["textDocument"]["uri"]        = data["uri"]
        params["textDocument"]["languageId"] = data["language_id"]
        params["textDocument"]["version"]    = data["version"]

        contentChanges      = params["contentChanges"][0]
        start               = contentChanges["range"]["start"]
        end                 = contentChanges["range"]["end"]
        start["line"]       = data["line"]
        start["character"]  = 0
        end["line"]         = data["line"]
        end["character"]    = data["column"]

        GLib.idle_add( self.send_notification, method, params )

    def _lsp_goto(self, data: dict):
        method      = data["method"]
        language_id = data["language_id"]
        uri         = data["uri"]
        line        = data["line"]
        column      = data["column"]

        GLib.idle_add( self.send_request, method, params )

    def _lsp_completion(self, data: dict):
        method = data["method"]
        params = completion_request["params"]

        params["textDocument"]["uri"]        = data["uri"]
        params["textDocument"]["languageId"] = data["language_id"]
        params["textDocument"]["version"]    = data["version"]
        params["position"]["line"]           = data["line"]
        params["position"]["character"]      = data["column"]


        GLib.idle_add( self.send_request, method, params )